@@ -8,13 +8,13 @@ import webbrowser
 import re
 import os
 from os.path import join, expanduser, abspath, isfile
-
 import requests
 import yaml
+from exceptions import StreamOutputError
 import tutum
 from tutum.api import auth
 from tutum.api import exceptions
-
+from packages import docker
 from tutumcli import utils
 from . import __version__
 
@@ -60,11 +60,11 @@ def login():
     username = raw_input("Username: ")
     password = getpass.getpass()
     try:
-        api_key = auth.get_apikey(username, password)
+        user, api_key = auth.get_auth(username, password)
         if api_key is not None:
             config = ConfigParser.ConfigParser()
             config.add_section(AUTH_SECTION)
-            config.set(AUTH_SECTION, USER_OPTION, username)
+            config.set(AUTH_SECTION, USER_OPTION, user)
             config.set(AUTH_SECTION, APIKEY_OPTION, api_key)
             with open(join(expanduser('~'), TUTUM_FILE), 'w') as cfgfile:
                 config.write(cfgfile)
@@ -87,7 +87,7 @@ def login():
         sys.exit(EXCEPTION_EXIT_CODE)
 
 
-def build(image_name, working_directory, quiet, nocache):
+def build(tag, working_directory, quiet, no_cache):
     try:
         directory = abspath(working_directory)
         dockerfile_path = join(directory, "Dockerfile")
@@ -124,11 +124,12 @@ def build(image_name, working_directory, quiet, nocache):
             utils.build_dockerfile(dockerfile_path, ports, cmd)
 
         docker_client = utils.get_docker_client()
-        output = docker_client.build(path=directory, tag=image_name, quiet=quiet, nocache=nocache, rm=True, stream=True)
-        for line in output:
-            if not quiet:
-                utils.print_stream_line(line)
-        print(image_name)
+        stream = docker_client.build(path=directory, tag=tag, quiet=quiet, nocache=no_cache, rm=True, stream=True)
+        try:
+            utils.stream_output(stream, sys.stdout)
+        except Exception as e:
+            print(e.message, file=sys.stderr)
+        print(tag)
     except Exception as e:
         print(e, file=sys.stderr)
         sys.exit(EXCEPTION_EXIT_CODE)
@@ -236,7 +237,7 @@ def apps_redeploy(identifiers, tag):
 
 
 def apps_run(image, name, container_size, target_num_containers, run_command, entrypoint, container_ports,
-             container_envvars, linked_to_applications, autorestart, autoreplace, autodestroy, roles, parallel):
+             container_envvars, linked_to_applications, autorestart, autoreplace, autodestroy, roles, sequential):
     try:
         ports = utils.parse_ports(container_ports)
         envvars = utils.parse_envvars(container_envvars)
@@ -245,7 +246,7 @@ def apps_run(image, name, container_size, target_num_containers, run_command, en
                                        entrypoint=entrypoint, container_ports=ports,
                                        container_envvars=envvars, linked_to_application=linked_to_applications,
                                        autorestart=autorestart, autoreplace=autoreplace, autodestroy=autodestroy,
-                                       roles=roles, parallel_deployment=parallel)
+                                       roles=roles, sequential_deployment=sequential)
         result = app.save()
         if result:
             print(app.uuid)
@@ -496,13 +497,25 @@ def images_push(name, public):
     def push_to_public(repository):
         print('Pushing %s to public registry ...' % repository)
 
-        output_stream = docker_client.push(repository, stream=True)
         output_status = NO_ERROR
-        for line in output_stream:
-            if 'status 401' in line.lower():
+        #tag a image to its name to check if the images exists
+        try:
+            docker_client.tag(name, name)
+        except Exception as e:
+            print(e, file=sys.stderr)
+            sys.exit(EXCEPTION_EXIT_CODE)
+        try:
+            stream = docker_client.push(repository, stream=True)
+            utils.stream_output(stream, sys.stdout)
+        except StreamOutputError as e:
+            if 'status 401' in e.message.lower():
                 output_status = AUTH_ERROR
-                continue
-            utils.print_stream_line(line)
+            else:
+                print(e, file=sys.stderr)
+                sys.exit(EXCEPTION_EXIT_CODE)
+        except Exception as e:
+            print(e.message, file=sys.stderr)
+            sys.exit(EXCEPTION_EXIT_CODE)
 
         if output_status == NO_ERROR:
             print('')
@@ -549,9 +562,15 @@ def images_push(name, public):
             print(e, file=sys.stderr)
             sys.exit(EXCEPTION_EXIT_CODE)
 
-        output_stream = docker_client.push(repository, stream=True)
-        for line in output_stream:
-            utils.print_stream_line(line)
+        stream = docker_client.push(repository, stream=True)
+        try:
+            utils.stream_output(stream, sys.stdout)
+        except docker.client.APIError as e:
+            print(e.explanation, file=sys.stderr)
+            sys.exit(EXCEPTION_EXIT_CODE)
+        except Exception as e:
+            print(e.message, file=sys.stderr)
+            sys.exit(EXCEPTION_EXIT_CODE)
         print('')
 
     docker_client = utils.get_docker_client()
